@@ -3,12 +3,14 @@ OC.L10N.register(
     {
     "None" : "Pas cap",
     "User" : "User",
+    "Name" : "Nom",
+    "Saved" : "Enregistrat",
     "Copy link" : "Copiar lo ligam",
     "Remove from favorites" : "Remove from favorites",
     "Add to favorites" : "Add to favorites",
     "Contacts" : "Contactes",
-    "Groups" : "Gropes",
     "Loading" : "Loading",
+    "Groups" : "Gropes",
     "Error" : "Error",
     "Password protect" : "Protegir per un senhal",
     "Back" : "Retorn",
@@ -17,21 +19,22 @@ OC.L10N.register(
     "Today" : "Uèi",
     "Yesterday" : "Ièr",
     "guest" : "convidat",
+    "Settings" : "Paramètres",
+    "Edit" : "Modificar",
+    "Dismiss" : "Regetar",
     "Share link" : "Partejar lo ligam",
     "Password protection" : "Password protection",
     "Enter a password" : "Enter a password",
-    "Edit" : "Modificar",
-    "Name" : "Nom",
-    "Saved" : "Enregistrat",
-    "Password" : "Senhal",
-    "Cancel" : "Anullar",
     "The password is wrong. Try again." : "Lo senhal es incorrècte. Ensajatz tornamai.",
+    "Password" : "Senhal",
     "Android app" : "Aplicacion Android",
     "iOS app" : "Aplicacion iOS",
-    "Share" : "Partejar",
+    "Copy" : "Copiar",
+    "Cancel" : "Anullar",
     "Change password" : "Cambiar de senhal",
     "Rename" : "Renomenar",
     "Send" : "Mandar",
+    "Share" : "Partejar",
     "Error while sharing" : "Error al moment del partiment"
 },
 "nplurals=2; plural=(n > 1);");
