@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 /**
  *
@@ -121,13 +122,13 @@ class ChatController extends AEnvironmentAwareController {
 	 *
 	 * @param string $message the message to send
 	 * @param string $actorDisplayName for guests
+	 * @param string $referenceId for the message to be able to later identify it again
 	 * @param int $replyTo Parent id which this message is a reply to
 	 * @return DataResponse the status code is "201 Created" if successful, and
 	 *         "404 Not found" if the room or session for a guest user was not
 	 *         found".
 	 */
-	public function sendMessage(string $message, string $actorDisplayName = '', int $replyTo = 0): DataResponse {
-
+	public function sendMessage(string $message, string $actorDisplayName = '', string $referenceId = '', int $replyTo = 0): DataResponse {
 		if ($this->userId === null) {
 			$actorType = 'guests';
 			$sessionId = $this->session->getSessionForRoom($this->room->getToken());
@@ -170,7 +171,7 @@ class ChatController extends AEnvironmentAwareController {
 		$creationDateTime = $this->timeFactory->getDateTime('now', new \DateTimeZone('UTC'));
 
 		try {
-			$comment = $this->chatManager->sendMessage($this->room, $this->participant, $actorType, $actorId, $message, $creationDateTime, $parent);
+			$comment = $this->chatManager->sendMessage($this->room, $this->participant, $actorType, $actorId, $message, $creationDateTime, $parent, $referenceId);
 		} catch (MessageTooLongException $e) {
 			return new DataResponse([], Http::STATUS_REQUEST_ENTITY_TOO_LARGE);
 		} catch (\Exception $e) {
@@ -304,7 +305,7 @@ class ChatController extends AEnvironmentAwareController {
 			$commentKey = $commentIdToIndex[$commentId];
 
 			// Parent is already parsed in the message list
-			if (!empty($commentIdToIndex[$parentId])) {
+			if (isset($commentIdToIndex[$parentId])) {
 				$parentKey = $commentIdToIndex[$parentId];
 				$messages[$commentKey]['parent'] = $messages[$parentKey];
 
@@ -359,10 +360,10 @@ class ChatController extends AEnvironmentAwareController {
 			 * So when you are still there, it will just have a wrong read
 			 * marker for the time until your next request starts, while it will
 			 * not update the value, when you actually left the chat already.
-			if ($setReadMarker === 1 && $lookIntoFuture) {
-				$this->participant->setLastReadMessage((int) $newLastKnown->getId());
-			}
-			*/
+			 * if ($setReadMarker === 1 && $lookIntoFuture) {
+			 * $this->participant->setLastReadMessage((int) $newLastKnown->getId());
+			 * }
+			 */
 		}
 
 		return $response;
