@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 /**
  * @copyright Copyright (c) 2016 Lukas Reschke <lukas@statuscode.ch>
@@ -30,6 +31,7 @@ use OCA\Circles\Api\v1\Circles;
 use OCA\Circles\Model\Member;
 use OCA\Talk\Chat\ChatManager;
 use OCA\Talk\Chat\MessageParser;
+use OCA\Talk\Config;
 use OCA\Talk\Events\UserEvent;
 use OCA\Talk\Exceptions\InvalidPasswordException;
 use OCA\Talk\Exceptions\ParticipantNotFoundException;
@@ -56,35 +58,36 @@ use OCP\IGroupManager;
 use OCP\IConfig;
 
 class RoomController extends AEnvironmentAwareController {
-
 	public const EVENT_BEFORE_ROOMS_GET = self::class . '::preGetRooms';
 
 	/** @var string|null */
-	private $userId;
+	protected $userId;
 	/** @var IAppManager */
-	private $appManager;
+	protected $appManager;
 	/** @var TalkSession */
-	private $session;
+	protected $session;
 	/** @var IUserManager */
-	private $userManager;
+	protected $userManager;
 	/** @var IGroupManager */
-	private $groupManager;
+	protected $groupManager;
 	/** @var Manager */
-	private $manager;
+	protected $manager;
 	/** @var GuestManager */
-	private $guestManager;
+	protected $guestManager;
 	/** @var ChatManager */
-	private $chatManager;
+	protected $chatManager;
 	/** @var IEventDispatcher */
-	private $dispatcher;
+	protected $dispatcher;
 	/** @var MessageParser */
-	private $messageParser;
+	protected $messageParser;
 	/** @var ITimeFactory */
-	private $timeFactory;
+	protected $timeFactory;
 	/** @var IL10N */
-	private $l10n;
+	protected $l10n;
 	/** @var IConfig */
-	private $config;
+	protected $config;
+	/** @var Config */
+	protected $talkConfig;
 
 	public function __construct(string $appName,
 								?string $UserId,
@@ -100,7 +103,8 @@ class RoomController extends AEnvironmentAwareController {
 								MessageParser $messageParser,
 								ITimeFactory $timeFactory,
 								IL10N $l10n,
-								IConfig $config) {
+								IConfig $config,
+								Config $talkConfig) {
 		parent::__construct($appName, $request);
 		$this->session = $session;
 		$this->appManager = $appManager;
@@ -115,6 +119,7 @@ class RoomController extends AEnvironmentAwareController {
 		$this->timeFactory = $timeFactory;
 		$this->l10n = $l10n;
 		$this->config = $config;
+		$this->talkConfig = $talkConfig;
 	}
 
 	protected function getTalkHashHeader(): array {
@@ -122,6 +127,13 @@ class RoomController extends AEnvironmentAwareController {
 			'X-Nextcloud-Talk-Hash' => sha1(
 				$this->config->getSystemValueString('version') . '#' .
 				$this->config->getAppValue('spreed', 'installed_version', '') . '#' .
+				$this->config->getAppValue('spreed', 'stun_servers', '') . '#' .
+				$this->config->getAppValue('spreed', 'turn_servers', '') . '#' .
+				$this->config->getAppValue('spreed', 'signaling_servers', '') . '#' .
+				$this->config->getAppValue('spreed', 'signaling_mode', '') . '#' .
+				$this->config->getAppValue('spreed', 'allowed_groups', '') . '#' .
+				$this->config->getAppValue('spreed', 'start_conversations', '') . '#' .
+				$this->config->getAppValue('spreed', 'has_reference_id', '') . '#' .
 				$this->config->getAppValue('theming', 'cachebuster', '1')
 		)];
 	}
@@ -178,12 +190,27 @@ class RoomController extends AEnvironmentAwareController {
 	}
 
 	/**
+	 * @param string $apiVersion
 	 * @param Room $room
 	 * @param Participant $currentParticipant
 	 * @return array
 	 * @throws RoomNotFoundException
 	 */
 	protected function formatRoom(Room $room, ?Participant $currentParticipant): array {
+		if ($this->getAPIVersion() === 2) {
+			return $this->formatRoomV2($room, $currentParticipant);
+		}
+
+		return $this->formatRoomV1($room, $currentParticipant);
+	}
+
+	/**
+	 * @param Room $room
+	 * @param Participant $currentParticipant
+	 * @return array
+	 * @throws RoomNotFoundException
+	 */
+	protected function formatRoomV1(Room $room, ?Participant $currentParticipant): array {
 		$roomData = [
 			'id' => $room->getId(),
 			'token' => $room->getToken(),
@@ -257,7 +284,7 @@ class RoomController extends AEnvironmentAwareController {
 		if ($roomData['notificationLevel'] === Participant::NOTIFY_DEFAULT) {
 			if ($currentParticipant->isGuest()) {
 				$roomData['notificationLevel'] = Participant::NOTIFY_NEVER;
-			} else if ($room->getType() === Room::ONE_TO_ONE_CALL) {
+			} elseif ($room->getType() === Room::ONE_TO_ONE_CALL) {
 				$roomData['notificationLevel'] = Participant::NOTIFY_ALWAYS;
 			} else {
 				$adminSetting = (int) $this->config->getAppValue('spreed', 'default_group_notification', Participant::NOTIFY_DEFAULT);
@@ -300,7 +327,7 @@ class RoomController extends AEnvironmentAwareController {
 		$cleanGuests = false;
 		$participantList = [];
 		$participants = $room->getParticipants();
-		uasort($participants, function(Participant $participant1, Participant $participant2) {
+		uasort($participants, function (Participant $participant1, Participant $participant2) {
 			return $participant2->getLastPing() - $participant1->getLastPing();
 		});
 
@@ -354,6 +381,145 @@ class RoomController extends AEnvironmentAwareController {
 		return $roomData;
 	}
 
+	/**
+	 * @param Room $room
+	 * @param Participant $currentParticipant
+	 * @return array
+	 * @throws RoomNotFoundException
+	 */
+	protected function formatRoomV2(Room $room, ?Participant $currentParticipant): array {
+		$roomData = [
+			'id' => $room->getId(),
+			'token' => $room->getToken(),
+			'type' => $room->getType(),
+			'name' => '',
+			'displayName' => '',
+			'objectType' => '',
+			'objectId' => '',
+			'participantType' => Participant::GUEST,
+			'participantFlags' => Participant::FLAG_DISCONNECTED,
+			'readOnly' => Room::READ_WRITE,
+			'hasPassword' => $room->hasPassword(),
+			'hasCall' => false,
+			'canStartCall' => false,
+			'lastActivity' => 0,
+			'lastReadMessage' => 0,
+			'unreadMessages' => 0,
+			'unreadMention' => false,
+			'isFavorite' => false,
+			'canLeaveConversation' => false,
+			'canDeleteConversation' => false,
+			'notificationLevel' => Participant::NOTIFY_NEVER,
+			'lastPing' => 0,
+			'sessionId' => '0',
+			'guestList' => '',
+			'lastMessage' => [],
+		];
+
+		if (!$currentParticipant instanceof Participant) {
+			return $roomData;
+		}
+
+		$lastActivity = $room->getLastActivity();
+		if ($lastActivity instanceof \DateTimeInterface) {
+			$lastActivity = $lastActivity->getTimestamp();
+		} else {
+			$lastActivity = 0;
+		}
+
+		$lobbyTimer = $room->getLobbyTimer();
+		if ($lobbyTimer instanceof \DateTimeInterface) {
+			$lobbyTimer = $lobbyTimer->getTimestamp();
+		} else {
+			$lobbyTimer = 0;
+		}
+
+		$roomData = array_merge($roomData, [
+			'name' => $room->getName(),
+			'displayName' => $room->getDisplayName($currentParticipant->getUser()),
+			'objectType' => $room->getObjectType(),
+			'objectId' => $room->getObjectId(),
+			'participantType' => $currentParticipant->getParticipantType(),
+			'participantFlags' => $currentParticipant->getInCallFlags(),
+			'readOnly' => $room->getReadOnly(),
+			'hasCall' => $room->getActiveSince() instanceof \DateTimeInterface,
+			'lastActivity' => $lastActivity,
+			'isFavorite' => $currentParticipant->isFavorite(),
+			'notificationLevel' => $currentParticipant->getNotificationLevel(),
+			'lobbyState' => $room->getLobbyState(),
+			'lobbyTimer' => $lobbyTimer,
+			'lastPing' => $currentParticipant->getLastPing(),
+			'sessionId' => $currentParticipant->getSessionId(),
+		]);
+
+		if ($roomData['notificationLevel'] === Participant::NOTIFY_DEFAULT) {
+			if ($currentParticipant->isGuest()) {
+				$roomData['notificationLevel'] = Participant::NOTIFY_NEVER;
+			} elseif ($room->getType() === Room::ONE_TO_ONE_CALL) {
+				$roomData['notificationLevel'] = Participant::NOTIFY_ALWAYS;
+			} else {
+				$adminSetting = (int) $this->config->getAppValue('spreed', 'default_group_notification', Participant::NOTIFY_DEFAULT);
+				if ($adminSetting === Participant::NOTIFY_DEFAULT) {
+					$roomData['notificationLevel'] = Participant::NOTIFY_MENTION;
+				} else {
+					$roomData['notificationLevel'] = $adminSetting;
+				}
+			}
+		}
+
+		if ($room->getLobbyState() === Webinary::LOBBY_NON_MODERATORS &&
+			!$currentParticipant->hasModeratorPermissions()) {
+			// No participants and chat messages for users in the lobby.
+			return $roomData;
+		}
+
+		$roomData['canStartCall'] = $currentParticipant->canStartCall();
+
+		$currentUser = $this->userManager->get($currentParticipant->getUser());
+		if ($currentUser instanceof IUser) {
+			$lastReadMessage = $currentParticipant->getLastReadMessage();
+			if ($lastReadMessage === -1) {
+				/*
+				 * Because the migration from the old comment_read_markers was
+				 * not possible in a programmatic way with a reasonable O(1) or O(n)
+				 * but only with O(user×chat), we do the conversion here.
+				 */
+				$lastReadMessage = $this->chatManager->getLastReadMessageFromLegacy($room, $currentUser);
+				$currentParticipant->setLastReadMessage($lastReadMessage);
+			}
+			$roomData['unreadMessages'] = $this->chatManager->getUnreadCount($room, $lastReadMessage);
+
+			$lastMention = $currentParticipant->getLastMentionMessage();
+			$roomData['unreadMention'] = $lastMention !== 0 && $lastReadMessage < $lastMention;
+			$roomData['lastReadMessage'] = $lastReadMessage;
+
+			$roomData['canDeleteConversation'] = $room->getType() !== Room::ONE_TO_ONE_CALL
+				&& $currentParticipant->hasModeratorPermissions(false);
+			$roomData['canLeaveConversation'] = !$roomData['canDeleteConversation']
+				|| ($room->getType() !== Room::ONE_TO_ONE_CALL && $room->getNumberOfParticipants() > 1);
+		}
+
+		// FIXME This should not be done, but currently all the clients use it to get the avatar of the user …
+		if ($room->getType() === Room::ONE_TO_ONE_CALL) {
+			$participants = $room->getParticipants();
+			foreach ($participants as $participant) {
+				$user = $this->userManager->get($participant->getUser());
+				if ($user instanceof IUser && $user->getUID() !== $currentParticipant->getUser()) {
+					$roomData['name'] = $user->getUID();
+				}
+			}
+		}
+
+		$lastMessage = $room->getLastMessage();
+		if ($lastMessage instanceof IComment) {
+			$roomData['lastMessage'] = $this->formatLastMessage($room, $currentParticipant, $lastMessage);
+		} else {
+			$roomData['lastMessage'] = [];
+		}
+
+		return $roomData;
+	}
+
 	/**
 	 * @param Room $room
 	 * @param Participant $participant
@@ -383,6 +549,15 @@ class RoomController extends AEnvironmentAwareController {
 	 * @return DataResponse
 	 */
 	public function createRoom(int $roomType, string $invite = '', string $roomName = '', string $source = ''): DataResponse {
+		if ($roomType !== Room::ONE_TO_ONE_CALL) {
+			/** @var IUser $user */
+			$user = $this->userManager->get($this->userId);
+
+			if ($this->talkConfig->isNotAllowedToCreateConversations($user)) {
+				return new DataResponse([], Http::STATUS_FORBIDDEN);
+			}
+		}
+
 		switch ($roomType) {
 			case Room::ONE_TO_ONE_CALL:
 				return $this->createOneToOneRoom($invite);
@@ -611,7 +786,6 @@ class RoomController extends AEnvironmentAwareController {
 	 * @return DataResponse
 	 */
 	public function setNotificationLevel(int $level): DataResponse {
-
 		if (!$this->participant->setNotificationLevel($level)) {
 			return new DataResponse([], Http::STATUS_BAD_REQUEST);
 		}
@@ -730,7 +904,7 @@ class RoomController extends AEnvironmentAwareController {
 			$this->room->addUsers([
 				'userId' => $newUser->getUID(),
 			]);
-		} else if ($source === 'groups') {
+		} elseif ($source === 'groups') {
 			$group = $this->groupManager->get($newParticipant);
 			if (!$group instanceof IGroup) {
 				return new DataResponse([], Http::STATUS_NOT_FOUND);
@@ -752,7 +926,7 @@ class RoomController extends AEnvironmentAwareController {
 			}
 
 			\call_user_func_array([$this->room, 'addUsers'], $participantsToAdd);
-		} else if ($source === 'circles') {
+		} elseif ($source === 'circles') {
 			if (!$this->appManager->isEnabledForUser('circles')) {
 				return new DataResponse([], Http::STATUS_BAD_REQUEST);
 			}
@@ -790,7 +964,7 @@ class RoomController extends AEnvironmentAwareController {
 			}
 
 			\call_user_func_array([$this->room, 'addUsers'], $participantsToAdd);
-		} else if ($source === 'emails') {
+		} elseif ($source === 'emails') {
 			$data = [];
 			if ($this->room->setType(Room::PUBLIC_CALL)) {
 				$data = ['type' => $this->room->getType()];
@@ -863,7 +1037,7 @@ class RoomController extends AEnvironmentAwareController {
 				&& $room->getNumberOfModerators() === 1) {
 				return new DataResponse([], Http::STATUS_BAD_REQUEST);
 			}
-		} else if ($room->getType() !== Room::CHANGELOG_CONVERSATION &&
+		} elseif ($room->getType() !== Room::CHANGELOG_CONVERSATION &&
 			$room->getNumberOfParticipants() === 1) {
 			$room->deleteRoom();
 			return new DataResponse();
