@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 /**
  * @copyright Copyright (c) 2016 Lukas Reschke <lukas@statuscode.ch>
@@ -38,6 +39,7 @@ use OCP\AppFramework\Http\DataResponse;
 use OCP\AppFramework\OCSController;
 use OCP\AppFramework\Utility\ITimeFactory;
 use OCP\EventDispatcher\IEventDispatcher;
+use OCP\Http\Client\IClientService;
 use OCP\IDBConnection;
 use OCP\IRequest;
 use OCP\IUser;
@@ -51,7 +53,9 @@ class SignalingController extends OCSController {
 	public const EVENT_BACKEND_SIGNALING_ROOMS = self::class . '::signalingBackendRoom';
 
 	/** @var Config */
-	private $config;
+	private $talkConfig;
+	/** @var \OCA\Talk\Signaling\Manager */
+	private $signalingManager;
 	/** @var TalkSession */
 	private $session;
 	/** @var Manager */
@@ -66,12 +70,15 @@ class SignalingController extends OCSController {
 	private $dispatcher;
 	/** @var ITimeFactory */
 	private $timeFactory;
+	/** @var IClientService */
+	private $clientService;
 	/** @var string|null */
 	private $userId;
 
 	public function __construct(string $appName,
 								IRequest $request,
-								Config $config,
+								Config $talkConfig,
+								\OCA\Talk\Signaling\Manager $signalingManager,
 								TalkSession $session,
 								Manager $manager,
 								IDBConnection $connection,
@@ -79,9 +86,11 @@ class SignalingController extends OCSController {
 								IUserManager $userManager,
 								IEventDispatcher $dispatcher,
 								ITimeFactory $timeFactory,
+								IClientService $clientService,
 								?string $UserId) {
 		parent::__construct($appName, $request);
-		$this->config = $config;
+		$this->talkConfig = $talkConfig;
+		$this->signalingManager = $signalingManager;
 		$this->session = $session;
 		$this->dbConnection = $connection;
 		$this->manager = $manager;
@@ -89,19 +98,106 @@ class SignalingController extends OCSController {
 		$this->userManager = $userManager;
 		$this->dispatcher = $dispatcher;
 		$this->timeFactory = $timeFactory;
+		$this->clientService = $clientService;
 		$this->userId = $UserId;
 	}
 
 	/**
 	 * @PublicPage
 	 *
+	 * @param string $token
+	 * @return DataResponse
+	 */
+	public function getSettings(string $token = ''): DataResponse {
+		try {
+			if ($token !== '') {
+				$room = $this->manager->getRoomForParticipantByToken($token, $this->userId);
+			} else {
+				// FIXME Soft-fail for legacy support in mobile apps
+				$room = null;
+			}
+		} catch (RoomNotFoundException $e) {
+			return new DataResponse([], Http::STATUS_NOT_FOUND);
+		}
+
+		$stun = [];
+		$stunServer = $this->talkConfig->getStunServer();
+		if ($stunServer) {
+			$stun[] = [
+				'url' => 'stun:' . $stunServer,
+			];
+		}
+
+		$turn = [];
+		$turnSettings = $this->talkConfig->getTurnSettings();
+		if (!empty($turnSettings['server'])) {
+			$protocols = explode(',', $turnSettings['protocols']);
+			foreach ($protocols as $proto) {
+				$turn[] = [
+					'url' => ['turn:' . $turnSettings['server'] . '?transport=' . $proto],
+					'urls' => ['turn:' . $turnSettings['server'] . '?transport=' . $proto],
+					'username' => $turnSettings['username'],
+					'credential' => $turnSettings['password'],
+				];
+			}
+		}
+
+		$signalingMode = $this->talkConfig->getSignalingMode();
+		$signaling = $this->signalingManager->getSignalingServerLinkForConversation($room);
+
+		return new DataResponse([
+			'signalingMode' => $signalingMode,
+			'userId' => $this->userId,
+			'hideWarning' => $signaling !== '' || $this->talkConfig->getHideSignalingWarning(),
+			'server' => $signaling,
+			'ticket' => $this->talkConfig->getSignalingTicket($this->userId),
+			'stunservers' => $stun,
+			'turnservers' => $turn,
+		]);
+	}
+
+	/**
 	 * Only available for logged in users because guests can not use the apps
 	 * right now.
 	 *
+	 * @param int $serverId
 	 * @return DataResponse
 	 */
-	public function getSettings(): DataResponse {
-		return new DataResponse($this->config->getSettings($this->userId));
+	public function getWelcomeMessage(int $serverId): DataResponse {
+		$signalingServers = $this->talkConfig->getSignalingServers();
+		if (empty($signalingServers) || !isset($signalingServers[$serverId])) {
+			return new DataResponse([], Http::STATUS_NOT_FOUND);
+		}
+
+		$url = rtrim($signalingServers[$serverId]['server'], '/');
+
+		if (strpos($url, 'wss://') === 0) {
+			$url = 'https://' . substr($url, 6);
+		}
+
+		if (strpos($url, 'ws://') === 0) {
+			$url = 'http://' . substr($url, 5);
+		}
+
+		$client = $this->clientService->newClient();
+		try {
+			$response = $client->get($url . '/api/v1/welcome', [
+				'verify' => (bool) $signalingServers[$serverId]['verify'],
+			]);
+
+			$body = $response->getBody();
+
+			$data = json_decode($body, true);
+			if (!is_array($data) || !isset($data['version'])) {
+				return new DataResponse(['error' => 'JSON_INVALID'], Http::STATUS_INTERNAL_SERVER_ERROR);
+			}
+
+			return new DataResponse($data);
+		} catch (\GuzzleHttp\Exception\ConnectException $e) {
+			return new DataResponse(['error' => 'CAN_NOT_CONNECT'], Http::STATUS_INTERNAL_SERVER_ERROR);
+		} catch (\Exception $e) {
+			return new DataResponse(['error' => $e->getCode()], Http::STATUS_INTERNAL_SERVER_ERROR);
+		}
 	}
 
 	/**
@@ -112,14 +208,13 @@ class SignalingController extends OCSController {
 	 * @return DataResponse
 	 */
 	public function signaling(string $token, string $messages): DataResponse {
-		$signaling = $this->config->getSignalingServers();
-		if (!empty($signaling)) {
+		if ($this->talkConfig->getSignalingMode() !== Config::SIGNALING_INTERNAL) {
 			return new DataResponse('Internal signaling disabled.', Http::STATUS_BAD_REQUEST);
 		}
 
 		$response = [];
 		$messages = json_decode($messages, true);
-		foreach($messages as $message) {
+		foreach ($messages as $message) {
 			$ev = $message['ev'];
 			switch ($ev) {
 				case 'message':
@@ -133,6 +228,15 @@ class SignalingController extends OCSController {
 					}
 					$decodedMessage['from'] = $message['sessionId'];
 
+					if ($decodedMessage['type'] === 'control') {
+						$room = $this->manager->getRoomForSession($this->userId, $message['sessionId']);
+						$participant = $room->getParticipantBySession($message['sessionId']);
+
+						if (!$participant->hasModeratorPermissions(false)) {
+							break;
+						}
+					}
+
 					$this->messages->addMessage($message['sessionId'], $decodedMessage['to'], json_encode($decodedMessage));
 
 					break;
@@ -149,8 +253,7 @@ class SignalingController extends OCSController {
 	 * @return DataResponse
 	 */
 	public function pullMessages(string $token): DataResponse {
-		$signaling = $this->config->getSignalingServers();
-		if (!empty($signaling)) {
+		if ($this->talkConfig->getSignalingMode() !== Config::SIGNALING_INTERNAL) {
 			return new DataResponse('Internal signaling disabled.', Http::STATUS_BAD_REQUEST);
 		}
 
@@ -176,7 +279,7 @@ class SignalingController extends OCSController {
 			// Query all messages and send them to the user
 			$data = $this->messages->getAndDeleteMessages($sessionId);
 			$messageCount = count($data);
-			$data = array_filter($data, function($message) {
+			$data = array_filter($data, function ($message) {
 				return $message['data'] !== 'refresh-participant-list';
 			});
 
@@ -274,7 +377,7 @@ class SignalingController extends OCSController {
 		if (empty($checksum)) {
 			return false;
 		}
-		$hash = hash_hmac('sha256', $random . $data, $this->config->getSignalingSecret());
+		$hash = hash_hmac('sha256', $random . $data, $this->talkConfig->getSignalingSecret());
 		return hash_equals($hash, strtolower($checksum));
 	}
 
@@ -293,7 +396,7 @@ class SignalingController extends OCSController {
 	 * servers.
 	 *
 	 * See sections "Backend validation" in
-	 * https://github.com/nextcloud/spreed/wiki/Spreed-Signaling-API
+	 * https://nextcloud-talk.readthedocs.io/en/latest/standalone-signaling-api-v1/#backend-validation
 	 *
 	 * @PublicPage
 	 *
@@ -336,7 +439,7 @@ class SignalingController extends OCSController {
 	private function backendAuth(array $auth): DataResponse {
 		$params = $auth['params'];
 		$userId = $params['userid'];
-		if (!$this->config->validateSignalingTicket($userId, $params['ticket'])) {
+		if (!$this->talkConfig->validateSignalingTicket($userId, $params['ticket'])) {
 			return new DataResponse([
 				'type' => 'error',
 				'error' => [
@@ -420,14 +523,19 @@ class SignalingController extends OCSController {
 
 		if ($action === 'join') {
 			$room->ping($userId, $sessionId, $this->timeFactory->getTime());
-		} else if ($action === 'leave') {
+		} elseif ($action === 'leave') {
 			if (!empty($userId)) {
 				$room->leaveRoom($userId, $sessionId);
-			} else if ($participant instanceof Participant) {
+			} elseif ($participant instanceof Participant) {
 				$room->removeParticipantBySession($participant, Room::PARTICIPANT_LEFT);
 			}
 		}
 
+		$permissions = ['publish-media', 'publish-screen'];
+		if ($participant instanceof Participant && $participant->hasModeratorPermissions(false)) {
+			$permissions[] = 'control';
+		}
+
 		$event = new SignalingEvent($room, $participant, $action);
 		$this->dispatcher->dispatch(self::EVENT_BACKEND_SIGNALING_ROOMS, $event);
 
@@ -437,6 +545,7 @@ class SignalingController extends OCSController {
 				'version' => '1.0',
 				'roomid' => $room->getToken(),
 				'properties' => $room->getPropertiesForSignaling((string) $userId),
+				'permissions' => $permissions,
 			],
 		];
 		if ($event->getSession()) {
@@ -478,5 +587,4 @@ class SignalingController extends OCSController {
 		];
 		return new DataResponse($response);
 	}
-
 }
