@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 /**
  * @copyright Copyright (c) 2016 Joas Schilling <coding@schilljs.com>
@@ -22,7 +23,6 @@ declare(strict_types=1);
 
 namespace OCA\Talk;
 
-
 use OCA\Talk\Chat\Changelog;
 use OCA\Talk\Chat\CommentsManager;
 use OCA\Talk\Events\CreateRoomTokenEvent;
@@ -34,6 +34,7 @@ use OCP\Comments\IComment;
 use OCP\Comments\NotFoundException;
 use OCP\DB\QueryBuilder\IQueryBuilder;
 use OCP\EventDispatcher\IEventDispatcher;
+use OCP\ICache;
 use OCP\IConfig;
 use OCP\IDBConnection;
 use OCP\IL10N;
@@ -43,7 +44,6 @@ use OCP\Security\IHasher;
 use OCP\Security\ISecureRandom;
 
 class Manager {
-
 	public const EVENT_TOKEN_GENERATE = self::class . '::generateNewToken';
 
 	/** @var IDBConnection */
@@ -140,6 +140,11 @@ class Manager {
 			]));
 		}
 
+		$assignedSignalingServer = $row['assigned_hpb'];
+		if ($assignedSignalingServer !== null) {
+			$assignedSignalingServer = (int) $assignedSignalingServer;
+		}
+
 		return new Room(
 			$this,
 			$this->db,
@@ -151,9 +156,10 @@ class Manager {
 			(int) $row['type'],
 			(int) $row['read_only'],
 			(int) $row['lobby_state'],
-			$row['token'],
-			$row['name'],
-			$row['password'],
+			$assignedSignalingServer,
+			(string) $row['token'],
+			(string) $row['name'],
+			(string) $row['password'],
 			(int) $row['active_guests'],
 			$activeSince,
 			$lastActivity,
@@ -201,6 +207,23 @@ class Manager {
 		}
 	}
 
+	public function resetAssignedSignalingServers(ICache $cache): void {
+		$query = $this->db->getQueryBuilder();
+		$query->select('*')
+			->from('talk_rooms')
+			->where($query->expr()->isNotNull('assigned_hpb'));
+
+		$result = $query->execute();
+		while ($row = $result->fetch()) {
+			$room = $this->createRoomObject($row);
+			if (!$room->hasActiveSessions()) {
+				$room->setAssignedSignalingServer(null);
+				$cache->remove($room->getToken());
+			}
+		}
+		$result->closeCursor();
+	}
+
 	/**
 	 * @param string $participant
 	 * @param bool $includeLastMessage
@@ -753,8 +776,8 @@ class Manager {
 		}
 
 		$roomName = implode(', ', $displayNames);
-		if (strlen($roomName) > 128) {
-			$roomName = substr($roomName, 120) . '…';
+		if (mb_strlen($roomName) > 64) {
+			$roomName = mb_substr($roomName, 0, 60) . '…';
 		}
 		return $roomName;
 	}
@@ -802,7 +825,6 @@ class Manager {
 	 * @throws \OutOfBoundsException
 	 */
 	protected function generateNewToken(IQueryBuilder $query, int $entropy, string $chars): string {
-
 		$event = new CreateRoomTokenEvent($entropy, $chars);
 		$this->dispatcher->dispatch(self::EVENT_TOKEN_GENERATE, $event);
 		try {
