@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 /**
  * @copyright Copyright (c) 2016 Lukas Reschke <lukas@statuscode.ch>
@@ -124,6 +125,8 @@ class Room {
 	private $readOnly;
 	/** @var int */
 	private $lobbyState;
+	/** @var int|null */
+	private $assignedSignalingServer;
 	/** @var \DateTime|null */
 	private $lobbyTimer;
 	/** @var string */
@@ -162,6 +165,7 @@ class Room {
 								int $type,
 								int $readOnly,
 								int $lobbyState,
+								?int $assignedSignalingServer,
 								string $token,
 								string $name,
 								string $password,
@@ -183,6 +187,7 @@ class Room {
 		$this->type = $type;
 		$this->readOnly = $readOnly;
 		$this->lobbyState = $lobbyState;
+		$this->assignedSignalingServer = $assignedSignalingServer;
 		$this->token = $token;
 		$this->name = $name;
 		$this->password = $password;
@@ -224,6 +229,10 @@ class Room {
 		}
 	}
 
+	public function getAssignedSignalingServer(): ?int {
+		return $this->assignedSignalingServer;
+	}
+
 	public function getToken(): string {
 		return $this->token;
 	}
@@ -453,7 +462,6 @@ class Room {
 	 * @return bool
 	 */
 	public function setActiveSince(\DateTime $since, bool $isGuest): bool {
-
 		if ($isGuest && $this->getType() === self::PUBLIC_CALL) {
 			$query = $this->db->getQueryBuilder();
 			$query->update('talk_rooms')
@@ -501,6 +509,19 @@ class Room {
 		return (bool) $query->execute();
 	}
 
+	public function setAssignedSignalingServer(?int $signalingServer): bool {
+		$query = $this->db->getQueryBuilder();
+		$query->update('talk_rooms')
+			->set('assigned_hpb', $query->createNamedParameter($signalingServer))
+			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+
+		if ($signalingServer !== null) {
+			$query->andWhere($query->expr()->isNull('assigned_hpb'));
+		}
+
+		return (bool) $query->execute();
+	}
+
 	/**
 	 * @param int $newType Currently it is only allowed to change between `self::GROUP_CALL` and `self::PUBLIC_CALL`
 	 * @return bool True when the change was valid, false otherwise
@@ -737,7 +758,6 @@ class Room {
 		$query->execute();
 
 		$this->dispatcher->dispatch(self::EVENT_AFTER_USER_REMOVE, $event);
-
 	}
 
 	/**
@@ -1106,6 +1126,23 @@ class Room {
 		return $participants;
 	}
 
+	/**
+	 * @return bool
+	 */
+	public function hasActiveSessions(): bool {
+		$query = $this->db->getQueryBuilder();
+		$query->select('room_id')
+			->from('talk_participants')
+			->where($query->expr()->eq('room_id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)))
+			->andWhere($query->expr()->neq('session_id', $query->createNamedParameter('0')))
+			->setMaxResults(1);
+		$result = $query->execute();
+		$row = $result->fetch();
+		$result->closeCursor();
+
+		return (bool) $row;
+	}
+
 	/**
 	 * @return string[]
 	 */
@@ -1205,6 +1242,7 @@ class Room {
 		if ($ignoreGuests) {
 			$query->andWhere($query->expr()->notIn('participant_type', $query->createNamedParameter([
 				Participant::GUEST,
+				Participant::GUEST_MODERATOR,
 				Participant::USER_SELF_JOINED,
 			], IQueryBuilder::PARAM_INT_ARRAY)));
 		}
